@@ -12,6 +12,7 @@ from __future__ import absolute_import, with_statement
 
 import contextlib
 import sys
+import logging
 import types
 
 from .. import io, scheduler, util
@@ -19,6 +20,8 @@ from .. import io, scheduler, util
 
 __all__ = ["patch", "unpatch", "patched", "patched_context"]
 
+log = logging.getLogger("greenhouse.emulation")
+
 
 def patched(module_name):
     """import and return a named module with patches applied locally only
@@ -152,6 +155,7 @@ def patch(*module_names):
     - select
     - signal
     - socket
+    - ssl
     - sys
     - thread
     - threading
@@ -169,11 +173,13 @@ def patch(*module_names):
 
         - ``subprocess`` works cooperatively with ``os`` and ``select`` patched
         - ``httplib``, ``urllib`` and ``urllib2`` will all operate
-          cooperatively with ``socket`` patched
+          cooperatively with ``socket`` and ``ssl`` patched
     """
     if not module_names:
         module_names = _patchers.keys()
 
+    log.info("monkey-patching in-place (%d modules)" % len(module_names))
+
     for module_name in module_names:
         if module_name not in _patchers:
             raise ValueError("'%s' is not greenhouse-patchable" % module_name)
@@ -203,6 +209,8 @@ def unpatch(*module_names):
     if not module_names:
         module_names = _standard.keys()
 
+    log.info("undoing monkey-patches in-place (%d modules)" % len(module_names))
+
     for module_name in module_names:
         if module_name not in _standard:
             raise ValueError("'%s' is not greenhouse-patchable" % module_name)
@@ -222,6 +230,9 @@ _patchers['select'] = select.patchers
 from . import socket
 _patchers['socket'] = socket.patchers
 
+from . import ssl
+_patchers['ssl'] = ssl.patchers
+
 from . import threading
 _patchers['thread'] = threading.thread_patchers
 _patchers['threading'] = threading.threading_patchers
@@ -246,7 +257,6 @@ if zmq.patchers:
 try:
     from . import dns
     _patchers['socket'].update(dns.socket_patchers)
-
 except ImportError:
     pass
 
@@ -259,11 +269,3 @@ for mod_name, patchers in _patchers.items():
     for attr_name, patcher in patchers.items():
         _standard[mod_name][attr_name] = getattr(module, attr_name, None)
 del mod_name, patchers, module, attr_name, patcher
-
-
-# pypy uses python code to build the interactive interpreter,
-# so we need to eagerly patch os.read and os.write for it
-import os
-if sys.subversion[0].lower() == 'pypy':
-    os.read = _patchers['os']['read']
-    os.write = _patchers['os']['write']
