@@ -12,12 +12,12 @@ __all__ = ["wait_callback"]
 def wait_callback(connection):
     """callback function suitable for ``psycopg2.set_wait_callback``
 
-    pass this function to ``psycopg2.set_wait_callack`` to force any blocking
-    operations from psycopg2 to only block the current coroutine, rather than
-    the entire thread or process
+    pass this function to ``psycopg2.extensions.set_wait_callack`` to force any
+    blocking operations from psycopg2 to only block the current coroutine, rather
+    than the entire thread or process
 
     to undo the change and return to normal blocking operation, use
-    `psycopg2.set_wait_callback(None)``
+    `psycopg2.extensions.set_wait_callback(None)``
     """
     while 1:
         state = connection.poll()
