@@ -1,9 +1,10 @@
 from __future__ import absolute_import
 
-from . import descriptor, files, ipc, sockets
+from . import descriptor, files, ipc, sockets, ssl
 
 
-__all__ = ["Socket", "File", "pipe", "stdin", "stdout", "stderr", "wait_fds"]
+__all__ = ["Socket", "File", "pipe", "stdin", "stdout", "stderr", "wait_fds",
+        "SSLSocket", "wrap_socket"]
 
 
 File = files.File
@@ -16,3 +17,5 @@ pipe = ipc.pipe
 Socket = sockets.Socket
 
 wait_fds = descriptor.wait_fds
+
+wrap_socket = SSLSocket = ssl.SSLSocket
