@@ -3,9 +3,7 @@ from __future__ import absolute_import, with_statement
 import errno
 import fcntl
 import os
-import socket
 import sys
-import weakref
 try:
     from cStringIO import StringIO
 except ImportError:
@@ -42,6 +40,8 @@ class FileBase(object):
     def read(self, size=-1):
         """read a number of bytes from the file and return it as a string
 
+        .. note:: this method will block if there is no data already available
+
         :param size:
             the maximum number of bytes to read from the file. < 0 means read
             the file to the end
@@ -85,6 +85,11 @@ class FileBase(object):
     def readline(self, max_len=-1):
         """read from the file until a newline is encountered
 
+        .. note::
+
+            this method will block if there isn't already a full line available
+            from the data source
+
         :param max_len: stop reading a single line after this many bytes
         :type max_len: int
 
@@ -136,6 +141,8 @@ class FileBase(object):
     def readlines(self, bufsize=-1):
         """reads the entire file, producing the lines one at a time
 
+        .. note:: this method will block until the data source is exhausted
+
         :param bufsize: the read buffer size to use
         :type bufsize: int
 
@@ -195,6 +202,7 @@ class File(FileBase):
     def __init__(self, name, mode='rb'):
         super(File, self).__init__()
         self.mode = mode
+        self.name = name
         self._closed = False
 
         # translate mode into the proper open flags
@@ -322,6 +330,11 @@ class File(FileBase):
         self._closed = True
         os.close(self._fileno)
 
+    @property
+    def closed(self):
+        "return True if the file descriptor is closed"
+        return self._closed
+
     def fileno(self):
         "get the file descriptor integer"
         return self._fileno
