@@ -4,7 +4,6 @@ import contextlib
 import errno
 import socket
 import sys
-import weakref
 
 from .. import scheduler, util
 from . import files
@@ -33,6 +32,18 @@ class Socket(object):
     def __init__(self, *args, **kwargs):
         self._sock = _InnerSocket(*args, **kwargs)
 
+    @property
+    def family(self):
+        return self._sock.family
+
+    @property
+    def proto(self):
+        return self._sock.proto
+
+    @property
+    def type(self):
+        return self._sock.type
+
     def close(self):
         """close the current connection on the socket
 
@@ -54,8 +65,10 @@ class Socket(object):
     def accept(self):
         """accept a connection on the host/port to which the socket is bound
 
-        If there is no connection attempt already queued, this method will
-        block until a connection is made
+        .. note::
+
+            if there is no connection attempt already queued, this method will
+            block until a connection is made
 
         :returns:
             a two-tuple of ``(socket, address)`` where the socket is connected,
@@ -79,8 +92,7 @@ class Socket(object):
     def connect(self, address):
         """initiate a new connection to a remote socket bound to an address
 
-        this method will block until the connection has been made; in the case
-        of TCP sockets this includes the TCP handshake
+        .. note:: this method will block until the connection has been made
 
         :param address:
             the address to which to initiate a connection, the format of which
@@ -188,7 +200,7 @@ class Socket(object):
     def recv(self, bufsize, flags=0):
         """receive data from the connection
 
-        this method will block until data is available to be read
+        .. note:: this method will block until data is available to be read
 
         see the unix manpage for ``recv(2)`` for more information
 
@@ -208,7 +220,7 @@ class Socket(object):
     def recv_into(self, buffer, bufsize=-1, flags=0):
         """receive data from the connection and place it into a buffer
 
-        this method will block until data is available to be read
+        .. note:: this method will block until data is available to be read
 
         :param buffer:
             a sized buffer object to receive the data (this is generally an
@@ -229,7 +241,7 @@ class Socket(object):
     def recvfrom(self, bufsize, flags=0):
         """receive data on a socket that isn't necessarily a 1-1 connection
 
-        this method will block until data is available to be read
+        .. note:: this method will block until data is available to be read
 
         :param bufsize:
             the maximum number of bytes to receive. fewer may be returned,
@@ -249,7 +261,7 @@ class Socket(object):
     def recvfrom_into(self, buffer, bufsize=-1, flags=0):
         """receive data on a non-TCP socket and place it in a buffer
 
-        this method will block until data is available to be read
+        .. note:: this method will block until data is available to be read
 
         :param buffer:
             a sized buffer object to receive the data (this is generally an
@@ -272,7 +284,7 @@ class Socket(object):
     def send(self, data, flags=0):
         """send data over the socket connection
 
-        this method may block if the socket's send buffer is full
+        .. note:: this method may block if the socket's send buffer is full
 
         :param data: the data to send
         :type data: str
@@ -290,7 +302,7 @@ class Socket(object):
     def sendall(self, data, flags=0):
         """send data over the connection, and keep sending until it all goes
 
-        this method may block if the socket's send buffer is full
+        .. note:: this method may block if the socket's send buffer is full
 
         :param data: the data to send
         :type data: str
@@ -304,7 +316,7 @@ class Socket(object):
     def sendto(self, data, *args):
         """send data to a particular address
 
-        this method may block if the socket's send buffer is full
+        .. note:: this method may block if the socket's send buffer is full
 
         :param data: the data to send
         :type data: str
@@ -380,10 +392,9 @@ class _InnerSocket(object):
     def __init__(self, *args, **kwargs):
         # wrap a basic socket or build our own
         sock = kwargs.pop('fromsock', None) or _socket(*args, **kwargs)
-        if hasattr(sock, "_sock"):
-            self._sock = sock._sock
-        else:
-            self._sock = sock
+        while hasattr(sock, "_sock"):
+            sock = sock._sock
+        self._sock = sock
 
         # copy over attributes
         self.family = self._sock.family
@@ -392,7 +403,8 @@ class _InnerSocket(object):
         self._fileno = self._sock.fileno()
 
         # some more housekeeping
-        self._timeout = sock.gettimeout()
+        tout = sock.gettimeout()
+        self._timeout = tout
         self._closed = False
 
         # make the underlying socket non-blocking
@@ -477,7 +489,6 @@ class _InnerSocket(object):
                 err = self.connect_ex(address)
                 if self._blocking and err in (
                         errno.EINPROGRESS, errno.EALREADY, errno.EWOULDBLOCK):
-                    sys.exc_clear()
                     if self._writable.wait(self.gettimeout()):
                         raise socket.timeout("timed out")
                     if scheduler.state.interrupted:
