@@ -1,7 +1,7 @@
 import bisect
 import collections
 import errno
-import operator
+import logging
 import sys
 import time
 import weakref
@@ -22,6 +22,8 @@ __all__ = ["pause", "pause_until", "pause_for", "schedule", "schedule_at",
 BTREE_ORDER = 64
 
 
+log = logging.getLogger("greenhouse.scheduler")
+
 state = type('GreenhouseState', (), {})()
 
 # from events that have triggered
@@ -127,28 +129,24 @@ except ImportError:
 
 
 def _hit_poller(timeout):
-    events = []
-    while 1:
-        try:
-            events = state.poller.poll(timeout)
-            break
-        except KeyboardInterrupt, exc:
-            # on Ctrl-C, wake up the main without killing the mainloop
-            state.to_raise[compat.main_greenlet] = exc
-            state.to_run.append(compat.main_greenlet)
-            return
-        except EnvironmentError, exc:
-            if exc.args[0] != errno.EINTR:
-                raise
+    try:
+        events = state.poller.poll(timeout)
+    except KeyboardInterrupt, exc:
+        # on Ctrl-C, wake up the main without killing the mainloop
+        state.to_raise[compat.main_greenlet] = exc
+        state.to_run.append(compat.main_greenlet)
+        return
+    except EnvironmentError, exc:
+        if exc.args[0] != errno.EINTR:
+            raise
 
-            # interrupted by a signal
-            if state.ignore_interrupts:
-                events = []
-            else:
-                state.interrupted = True
-                events = [(fd, state.poller.ERRMASK)
-                        for fd in state.poller._registry.iterkeys()]
-            break
+        # interrupted by a signal
+        if state.ignore_interrupts:
+            events = []
+        else:
+            state.interrupted = True
+            events = [(fd, state.poller.ERRMASK)
+                    for fd in state.poller._registry.iterkeys()]
 
     for fd, eventmap in events:
         readables = []
@@ -164,7 +162,9 @@ def _hit_poller(timeout):
                 if writable:
                     writables.append(writable)
 
-        map(callbackpairs.pop, removals[::-1])
+        for index in removals[::-1]:
+            callbackpairs.pop(index)
+
         if not callbackpairs:
             state.descriptormap.pop(fd)
 
@@ -242,13 +242,18 @@ def greenlet(func, args=(), kwargs=None):
     return compat.greenlet(target, state.mainloop)
 
 def pause():
-    "pause and reschedule the current greenlet and switch to the next"
+    """pause and reschedule the current greenlet and switch to the next
+
+    .. note:: this method blocks for a short period of time
+    """
     schedule(compat.getcurrent())
     state.mainloop.switch()
 
 def pause_until(unixtime):
     """pause and reschedule the current greenlet until a set time
 
+    .. note:: this method will block the current greenlet
+
     :param unixtime: the unix timestamp of when to bring this greenlet back
     :type unixtime: int or float
     """
@@ -258,6 +263,8 @@ def pause_until(unixtime):
 def pause_for(secs):
     """pause and reschedule the current greenlet for a set number of seconds
 
+    .. note:: this method will block the current greenlet
+
     :param secs: number of seconds to pause
     :type secs: int or float
     """
@@ -497,7 +504,7 @@ def end(target):
         schedule(target)
         state.to_raise[target] = compat.GreenletExit()
 
-def _remove_from_timedout(waketime, glet):
+def _remove_timer(waketime, glet):
     if state.timed_paused.remove(waketime, glet):
         return True
 
@@ -520,15 +527,14 @@ def mainloop():
 
         if not state.to_run:
             _hit_poller(0)
-
-        while not state.to_run:
-            # if there are timed-paused greenlets, we can
-            # just wait until the first of them wakes up
-            if state.timed_paused:
-                until = state.timed_paused.first()[0] + 0.001
-                _hit_poller(until - time.time())
-            else:
-                _hit_poller(None)
+            while not state.to_run:
+                # if there are timed-paused greenlets, we can
+                # just wait until the first of them wakes up
+                if state.timed_paused:
+                    until = state.timed_paused.first()[0] + 0.001
+                    _hit_poller(until - time.time())
+                else:
+                    _hit_poller(None)
 
         prev = target
         target = state.to_run.popleft()
@@ -668,6 +674,7 @@ def global_exception_handler(handler):
     """
     if not hasattr(handler, "__call__"):
         raise TypeError("exception handlers must be callable")
+    log.info("setting a new global exception handler")
 
     state.global_exception_handlers.append(weakref.ref(handler))
 
@@ -687,6 +694,7 @@ def remove_global_exception_handler(handler):
         cb = cb()
         if cb is not None and cb is handler:
             state.global_exception_handlers.pop(i)
+            log.info("removing a global exception handler")
             return True
     return False
 
@@ -714,6 +722,8 @@ def local_exception_handler(handler=None, coro=None):
     if coro is None:
         coro = compat.getcurrent()
 
+    log.info("setting a new coroutine local exception handler")
+
     state.local_exception_handlers.setdefault(coro, []).append(
             weakref.ref(handler))
 
@@ -736,6 +746,7 @@ def remove_local_exception_handler(handler, coro=None):
         cb = cb()
         if cb is not None and cb is handler:
             state.local_exception_handlers[coro].pop(i)
+            log.info("removing a coroutine local exception handler")
             return True
     return False
 
@@ -755,6 +766,7 @@ def global_hook(handler):
     if not hasattr(handler, "__call__"):
         raise TypeError("trace hooks must be callable")
 
+    log.info("setting a new global hook callback")
     state.global_hooks.append(weakref.ref(handler))
 
     return handler
@@ -773,6 +785,7 @@ def remove_global_hook(handler):
         cb = cb()
         if cb is not None and cb is handler:
             state.global_hooks.pop(i)
+            log.info("removing a global hook callback")
             return True
     return False
 
@@ -804,6 +817,8 @@ def local_incoming_hook(handler=None, coro=None):
     if coro is None:
         coro = compat.getcurrent()
 
+    log.info("setting a coroutine incoming local hook callback")
+
     state.local_to_hooks.setdefault(coro, []).append(
             weakref.ref(handler))
 
@@ -825,6 +840,7 @@ def remove_local_incoming_hook(handler, coro=None):
     for i, cb in enumerate(state.local_to_hooks.get(coro, [])):
         cb = cb()
         if cb is not None and cb is handler:
+            log.info("removing a coroutine incoming local hook callback")
             state.local_to_hooks[coro].pop(i)
             return True
     return False
@@ -855,6 +871,8 @@ def local_outgoing_hook(handler=None, coro=None):
     if coro is None:
         coro = compat.getcurrent()
 
+    log.info("setting a coroutine local outgoing hook callback")
+
     state.local_from_hooks.setdefault(coro, []).append(
             weakref.ref(handler))
 
@@ -876,6 +894,7 @@ def remove_local_outgoing_hook(handler, coro=None):
     for i, cb in enumerate(state.local_from_hooks.get(coro, [])):
         cb = cb()
         if cb is not None and cb is handler:
+            log.info("removing a coroutine outgoing local hook callback")
             state.local_from_hooks[coro].pop(i)
             return True
     return False
@@ -892,6 +911,7 @@ def set_ignore_interrupts(flag=True):
         whether to turn EINTR exceptions off (``True``) or on (``False``)
     :type flag: bool
     """
+    log.info("setting ignore_interrupts to %r" % flag)
     state.ignore_interrupts = bool(flag)
 
 def reset_poller(poll=None):
@@ -900,3 +920,4 @@ def reset_poller(poll=None):
     this is only really a good idea in the new child process after a fork(2).
     """
     state.poller = poll or poller.best()
+    log.info("resetting fd poller, using %s" % type(state.poller).__name__)
